const { SyncFile, File } = require("ada-util");
const Path = require("path");
const mysql = require("mysql2");
const config = require("./../../../config");
const Template = require("art-template");

const TypesMap = {
    'TINYINT': 'Types.INTEGER',
    'SMALLINT': 'Types.INTEGER',
    'MEDIUMINT': 'Types.INTEGER',
    'INT': 'Types.INTEGER',
    'INTEGER': 'Types.INTEGER',
    'BIGINT': 'Types.INTEGER',
    'FLOAT': 'Types.TypeNumber',
    'DOUBLE': 'Types.TypeNumber',
    'DECIMAL': 'Types.TypeNumber',
    'DATE': 'Types.DATE',
    'TIME': 'Types.DATE',
    'YEAR': 'Types.DATE',
    'DATETIME': 'Types.DATE',
    'TIMESTAMP': 'Types.DATE',
    'CHAR': 'Types.STRING',
    'VARCHAR': 'Types.STRING',
    'TINYBLOB': 'Types.BLOB',
    'TINYTEXT': 'Types.STRING',
    'BLOB': 'Types.BLOB',
    'TEXT': 'Types.STRING',
    'MEDIUMBLOB': 'Types.BLOB',
    'MEDIUMTEXT': 'Types.STRING',
    'LONGBLOB': 'Types.BLOB',
    'LONGTEXT': 'Types.STRING'
}

const BoostUtil = {
    getConfig() {
        return config;
    },
    getTableInfo() {
        return new Promise((resolve, reject) => {
            let { host, user, port, password, database, charset } = this.getConfig().get('generate');
            const connection = mysql.createConnection({
                host, user, port, password, database, charset
            });
            connection.query('show tables', (err, results) => {
                if (!err) {
                    let tables = results.map(a => a[`Tables_in_${database}`]).filter(a => !!a), map = {};
                    return tables.reduce((a, tableName) => {
                        return a.then(() => {
                            return new Promise((resolve, reject) => {
                                connection.query('show full columns from `' + tableName + '`', (err, result) => {
                                    if (err) {
                                        reject(err);
                                    } else {
                                        result = result.map(a => {
                                            return {
                                                Field: a.Field,
                                                Type: a.Type,
                                                Collation: a.Collation,
                                                Null: a.Null,
                                                Key: a.Key,
                                                Default: a.Default,
                                                Extra: a.Extra,
                                                Privileges: a.Privileges,
                                                Comment: a.Comment
                                            }
                                        });
                                        map[tableName] = result;
                                        resolve(result);
                                    }
                                });
                            });
                        });
                    }, Promise.resolve()).then(() => {
                        connection.close();
                        resolve(map);
                    });
                } else {
                    reject(err);
                }
            });
        });
    },
    getFileContent(type) {
        let path = Path.resolve(__dirname, `./template/${type}.tpl`);
        return new SyncFile(path).read();
    },
    parseTemplate(type, info) {
        return Template.render(this.getFileContent(type), info);
    },
    getCamelName(name) {
        let camelName = name.replace(/_[a-zA-Z]/g, str => {
            return str.substring(1).toUpperCase();
        });
        let upperCamelName = camelName.substring(0, 1).toUpperCase() + camelName.substring(1, camelName.length);
        return { camelName, upperCamelName }
    },
    outputConsumer() {
        let { consumer, output } = this.getConfig().get('generate');
        if (output && consumer && consumer.length > 0) {
            return consumer.reduce((a, b) => {
                return a.then(() => {
                    let name = b;
                    let camelName = name.replace(/-[a-zA-Z]/g, str => {
                        return str.substring(1).toUpperCase();
                    });
                    let upperCamelName = camelName.substring(0, 1).toUpperCase() + camelName.substring(1, camelName.length);
                    let content = this.parseTemplate('consumer', { name, camelName, upperCamelName });
                    return new File(Path.resolve(output, `./consumer/${name}.js`)).write(content);
                });
            }, Promise.resolve());
        }else{
            return Promise.resolve();
        }
    },
    output() {
        return this.outputConsumer().then(() => {
            let { output, name, host, user, port } = info = this.getConfig().get('generate');
            if (host && user && port) {
                return this.getTableInfo().then(tableMap => {
                    return Reflect.ownKeys(tableMap).map(tableName => {
                        let { camelName, upperCamelName } = this.getCamelName(tableName);
                        let fields = {};
                        tableMap[tableName].forEach(item => {
                            let n = item.Type.split(/\(|\)/);
                            let type = TypesMap[n.shift().toUpperCase()] || 'Types.STRING';
                            let length = n.shift() || null;
                            let valid = {};
                            if (item.Null === 'NO') {
                                valid.required = true;
                            }
                            if (length) {
                                valid.max = length.indexOf(",") === -1 ? length * 1 : length;
                            }
                            let validStr = JSON.stringify(valid).replace(/"[\s\S]+":/g, str => {
                                return str.replace(/"/g, '');
                            });
                            if (item.Key === "PRI") {
                                fields['id'] = {
                                    prime: true,
                                    type,
                                    field: item.Field,
                                    length,
                                    notNull: item.Null,
                                    defaultValue: item.Default,
                                    valid,
                                    validStr
                                };
                            } else {
                                let { camelName } = this.getCamelName(item.Field);
                                fields[camelName] = {
                                    field: item.Field,
                                    type,
                                    length,
                                    notNull: item.Null,
                                    defaultValue: item.Default,
                                    valid,
                                    validStr
                                };
                            }
                        });
                        return {
                            tableName,
                            url: tableName.replace(/_/g, ''),
                            tableCamelName: camelName,
                            upperTableCamelName: upperCamelName,
                            fields
                        }
                    }).reduce((a, table) => {
                        return a.then(() => {
                            let { tableName, tableCamelName, upperTableCamelName, fields, url } = table;
                            return Promise.resolve().then(() => {
                                let content = this.parseTemplate('controller', { tableName, tableCamelName, upperTableCamelName, fields, url, name });
                                return new File(Path.resolve(output, `./controller/${tableName}.js`)).write(content);
                            }).then(() => {
                                let content = this.parseTemplate('model', { tableName, tableCamelName, upperTableCamelName, fields, url, name });
                                return new File(Path.resolve(output, `./model/${tableName}.js`)).write(content);
                            }).then(() => {
                                let content = this.parseTemplate('service', { tableName, tableCamelName, upperTableCamelName, fields, url, name });
                                return new File(Path.resolve(output, `./service/${tableName}.js`)).write(content);
                            });
                        });
                    }, Promise.resolve());
                });
            }
        });
    }
};

module.exports = BoostUtil;