const { File } = require("ada-util");
const Path = require("path");
const Controller = require("./controller");
const Consumer = require("./consumer");
const Dao = require("./dao/base");
const Service = require("./service");
const Model = require("./model");
const Datasource = require("./datasource/base");
const MysqlDatasource = require("./datasource");
const MysqlDao = require("./dao");
const Types = require("./types");
const BoostContext = require("./context");
const DatasourceManager = require("./datasource/manager");
const ConsumerManager = require("./consumer/manager");
const ControllerManager = require("./controller/manager");
const UUID = require("uuid/v4");
const logger = require("./../logger");
const Result = require("./../result");
const config = require("./../config");
const LocalStore = require("./localstore");

const Boost = {
    boot({ source, server, service, serviceVerifier, channel }) {
        BoostContext.setContextInfo({ service, serviceVerifier, channel, server });
        return new File(source).getAllSubFilePaths().then(paths => {
            paths.filter(path => Path.extname(path) === ".js").forEach(path => {
                let a = require(path), clazz = a ? a.default || a : {};
                if (clazz.prototype instanceof Controller) {
                    ControllerManager.set(clazz);
                } else if (clazz.prototype instanceof Datasource) {
                    DatasourceManager.set(clazz);
                } else if (clazz.prototype instanceof Consumer) {
                    ConsumerManager.set(clazz);
                }
            });
        }).then(() => {
            return Promise.resolve().then(() => {
                if (server.getDatabaseOption) {
                    return DatasourceManager.init();
                }
            }).then(() => {
                ConsumerManager.getObserveNames().forEach(name => {
                    logger.get('boost').debug(`consumer observe [${name}]`);
                    channel.observe(name, info => {
                        ConsumerManager.excuteObserve({ name, info, localStore: new LocalStore() });
                    });
                });
                ConsumerManager.getWatchNames().forEach(name => {
                    logger.get('boost').debug(`consumer watch [${name}]`);
                    channel.watch(name, info => {
                        ConsumerManager.excuteWatch({ name, info, localStore: new LocalStore() });
                    });
                });
                if (server.use) {
                    server.use((ctx, next) => {
                        logger.get('boost').debug(`request [${ctx.method.toUpperCase()} => ${ctx.path}]`);
                        if (!config.get('gateway')) {
                            let tokenStr = ctx.request.header.authorization, token = '';
                            if (tokenStr) {
                                token = tokenStr.split(" ").pop();
                            }
                            if (token && serviceVerifier.verify(token)) {
                                return ControllerManager.excute({
                                    path: `/${ctx.method.toLowerCase()}${ctx.path}`, localStore: new LocalStore(), ctx, next
                                });
                            } else {
                                ctx.set("Content-Type", "application/json");
                                ctx.response.body = Result.getUnauthoriedResult().getResponseData();
                            }
                        } else {
                            return ControllerManager.excute({
                                path: `/${ctx.method.toLowerCase()}${ctx.path}`, localStore: new LocalStore(), ctx, next
                            });
                        }
                    });
                }
            });
        }).catch(e => {
            logger.get("boost").error(`boot | ${e.stack}`);
        });
    },
    updateDatasource() {
        DatasourceManager.update();
    }
}

function uuid() {
    return UUID().replace(/-/g, '');
}

module.exports = {
    Service,
    Controller,
    Consumer,
    Dao,
    Model,
    MysqlDatasource,
    MysqlDao,
    Types,
    Boost,
    uuid
};