const logger = require("./../logger");
const Metadata = require("./../metadata");
const Dao = require("./dao/base");
const Datasource = require("./datasource/base");
const DatasourceMap = require("./datasource/manager");
const BoostContext = require("./context");
const Service = require("./service");

const BoostProvider = {
    getService(targetClass, localStore) {
        let targetInstance = new targetClass();
        return new Proxy(targetInstance, {
            get(target, name) {
                if (typeof target[name] === 'function') {
                    let fields = Metadata.getPropertyMetadata(target.__proto__.constructor.prototype);
                    let methods = Metadata.getMethodMetadata(target.__proto__.constructor.prototype);
                    let action = methods[name] || {}, { transaction = false } = action;
                    target._localStore = localStore;
                    return (...args) => {
                        let promise = Promise.resolve(), map = new Map();
                        localStore.log({ service: name });
                        Reflect.ownKeys(fields).filter(a => !!fields[a].dao && fields[a].dao.prototype instanceof Dao).forEach(keyName => {
                            let daoClass = fields[keyName].dao, { datasource } = Metadata.getClassMetadata(daoClass.prototype) || {};
                            if (datasource && datasource.prototype instanceof Datasource) {
                                let { database } = Metadata.getClassMetadata(datasource.prototype) || {};
                                if (database) {
                                    promise = promise.then(() => {
                                        let _datasource = DatasourceMap.get(database);
                                        return _datasource.getConnection().then(connection => {
                                            target[keyName] = new daoClass(connection);
                                            target[keyName]._localStore = localStore;
                                            map.set(_datasource, connection);
                                        });
                                    });
                                }
                            }
                        });
                        return promise.then(() => {
                            let ps = Promise.resolve();
                            localStore.log({ hasTransaction: !!transaction });
                            if (transaction) {
                                ps = ps.then(() => {
                                    return [...map.values()].reduce((a, connection) => {
                                        return a.then(() => connection.beginTransaction());
                                    }, Promise.resolve()).then(() => {
                                        localStore.log({ startTransaction: true });
                                    });
                                });
                            }
                            return ps.then(() => {
                                return new Promise((resolve, reject) => {
                                    let errorInfo = null, resultInfo = null;
                                    Promise.resolve().then(() => target.beforeExcute(action)).then(() => {
                                        return Promise.resolve().then(() => target[name].apply(target, args));
                                    }).then(a => {
                                        return Promise.resolve().then(() => target.afterExcute({ result: a, action }));
                                    }).then(a => {
                                        resultInfo = a;
                                        if (transaction) {
                                            return [...map.values()].reduce((a, connection) => {
                                                return a.then(() => connection.commit());
                                            }, Promise.resolve()).then(() => {
                                                localStore.log({ commitTransaction: true });
                                            });
                                        }
                                    }).catch(err => {
                                        errorInfo = err;
                                        logger.get('boost').error(err.stack);
                                        localStore.log({ serviceError: err });
                                        if (transaction) {
                                            return [...map.values()].reduce((a, connection) => {
                                                return a.then(() => connection.rollback());
                                            }, Promise.resolve()).then(() => {
                                                localStore.log({ rollbackTransaction: true });
                                            });
                                        }
                                    }).finally(a => {
                                        map.forEach((connection, datasource) => {
                                            datasource.releaseConnection(connection);
                                        });
                                        if (errorInfo) {
                                            reject(errorInfo);
                                        } else {
                                            resolve(resultInfo);
                                        }
                                    });
                                });
                            });
                        });
                    }
                } else {
                    return target[name];
                }
            }
        });
    },
    getEntity(targetClass, localStore) {
        let targetObject = new targetClass();
        let fields = Metadata.getPropertyMetadata(targetClass.prototype) || {};
        let _service = [], _channel = [], _remoteService = [], _remoteVerify = [];
        targetObject._localStore = localStore;
        Reflect.ownKeys(fields).forEach(propName => {
            if (!!fields[propName].service && fields[propName].service.prototype instanceof Service) {
                _service.push(propName);
            }
            if (!!fields[propName].channel) {
                _channel.push(propName);
            }
            if (!!fields[propName].remoteService) {
                _remoteService.push(propName);
            }
            if (!!fields[propName].remoteVerify) {
                _remoteVerify.push(propName);
            }
        });
        _service.forEach(propName => {
            targetObject[propName] = this.getService(fields[propName].service, localStore);
        });
        _channel.forEach(propName => {
            targetObject[propName] = BoostContext.info.channel;
        });
        _remoteService.forEach(propName => {
            targetObject[propName] = BoostContext.info.service;
        });
        _remoteVerify.forEach(propName => {
            targetObject[propName] = BoostContext.info.serviceVerifier;
        });
        return targetObject;
    }
}

module.exports = BoostProvider;